@@ -1,19 +1,19 @@
 #
 # This file is part of Config-Model
 #
-# This software is Copyright (c) 2010 by Dominique Dumont, Krzysztof Tyszecki.
+# This software is Copyright (c) 2011 by Dominique Dumont, Krzysztof Tyszecki.
 #
 # This is free software, licensed under:
 #
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
 package Config::Model;
-BEGIN {
-  $Config::Model::VERSION = '1.226';
+{
+  $Config::Model::VERSION = '1.260';
 }
-use Moose ;
-use Moose::Util::TypeConstraints;
-use MooseX::StrictConstructor ;
+use Any::Moose ;
+use Any::Moose '::Util::TypeConstraints';
+use Any::Moose 'X::StrictConstructor' ;
 
 use Carp;
 use Storable ('dclone') ;
@@ -21,6 +21,7 @@ use Data::Dumper ();
 use Log::Log4perl 1.11 qw(get_logger :levels);
 use Config::Model::Instance ;
 use Hash::Merge qw/merge/ ;
+use File::Path qw/make_path/;
 
 # this class holds the version number of the package
 use vars qw(@status @level @experience_list %experience_index
@@ -94,7 +95,7 @@ Config::Model - Create tools to validate, migrate and edit configuration files
 
 =head1 VERSION
 
-version 1.226
+version 1.260
 
 =head1 SYNOPSIS
 
@@ -151,8 +152,8 @@ version 1.226
  $ mkdir -p lib/Config/Model/models/
  $ config-model-edit -model MiniModel -save \
    class:MiniModel element:foo type=leaf value_type=uniline - \
-   element:bar type=leaf value_type=uniline - \
-   element:baz type=leaf value_type=uniline - \
+                   element:bar type=leaf value_type=uniline - \
+                   element:baz type=leaf value_type=uniline - \
    read_config:0 backend=IniFile file=mini.ini config_dir=. auto_create=1 - - -
  $ config-edit -model MiniModel -model_dir lib/Config/Model/models/ -ui none bar=BARV foo=FOOV baz=BAZV
  $ cat mini.ini
@@ -192,7 +193,7 @@ only upgrade and migration specifications are required
 
 =item *
 
-unknow parameters can be accepted 
+unknown parameters can be accepted 
 
 =back
 
@@ -251,7 +252,7 @@ and work fine, but they have their set of drawbacks.
 Usually, the validation of configuration data is done with a script
 which performs semantic validation and often ends up being quite
 complex (e.g. 2500 lines for Debian's xserver-xorg.config script which
-handles xorg.conf file).
+handles C<xorg.conf> file).
 
 In most cases, the configuration model is expressed in instructions
 (whatever programming language is used) and interspersed with a lot of
@@ -284,7 +285,7 @@ enum like type, default value ...)
 
 =item *
 
-The targeted audience (beginer, advanced, master)
+The targeted audience (beginner, advanced, master)
 
 =item *
 
@@ -307,7 +308,7 @@ project.
 
 =item *
 
-Beginners will not see advanced parameters (advanved and master
+Beginners will not see advanced parameters (advanced and master
 parameters are hidden from beginners)
 
 =item *
@@ -360,7 +361,7 @@ to another, people who want to use this framework may have to
 provide a dedicated parser/writer.
 
 To help with this task, this project provides writer/parsers for common
-format: ini style file and perl file. With the additional
+format: INI style file and perl file. With the additional
 Config::Model::Backend::Augeas, Augeas library can be used to read and
 write some configuration files. See http://augeas.net for more
 details.
@@ -399,7 +400,7 @@ L<Config::Model::Cookbook::CreateModelFromDoc>
 
 =item *
 
-L<Config::Model::Manual::ModelCreationIntroductionAdvanced>
+L<Config::Model::Manual::ModelCreationAdvanced>
 
 =back
 
@@ -487,7 +488,7 @@ List or hash of node or value elements
 
 =back
 
-By declaring a set of configuration classes and refering them in node
+By declaring a set of configuration classes and referring them in node
 element, you will shape the structure of your configuration tree.
 
 The structure of the configuration data must be based on a tree
@@ -783,9 +784,10 @@ L<Config::Model::Itself> model editor.
 =cut
 
 my @legal_params = qw/element experience status description summary level
-                      config_dir generated_by class_description
+                      config_dir
                       read_config read_config_dir write_config
                       write_config_dir accept/;
+my @static_legal_params = qw/class_description copyright author license generated_by/ ;                      
 
 sub create_config_class {
     my $self=shift ;
@@ -800,7 +802,7 @@ sub create_config_class {
         Config::Model::Exception::ModelDeclaration->throw
             (
              error=> "create_config_class: attempt to clobber $config_class_name".
-             "config class name "
+             " config class name "
             );
     }
 
@@ -868,7 +870,7 @@ sub _create_config_class {
         (
          error=> "create class $config_class_name: unknown ".
          "parameter '" . join("', '",@left_params)."', expected '".
-         join("', '",@legal_params,qw/class_description/)."'"
+         join("', '",@legal_params, @static_legal_params)."'"
         )
           if @left_params ;
 
@@ -932,7 +934,7 @@ sub check_class_parameters {
         # this parameter is filled by class generated by a program. It may
         # be used to avoid interactive edition of a generated model
         'generated_by',
-        'class_description',
+        qw/class_description author copyright license/
     ) ;
 
     foreach my $info (@info_to_move) {
@@ -944,11 +946,22 @@ sub check_class_parameters {
     my @accept_list ;
     my %accept_hash ;
     my $accept_info = delete $raw_model->{'accept'} || [] ;
-    foreach my $accept_item (@$accept_info) {
-        my $name_match = delete $accept_item->{name_match} || '.*';
+    while (@$accept_info) {
+        my $name_match = shift @$accept_info ; # should be a regexp
+
+        # handle legacy
+        if (ref $name_match) {
+            my $implicit = defined $name_match->{name_match} ? '' :'implicit ';
+            unshift @$accept_info, $name_match; # put data back in list
+            $name_match  = delete $name_match->{name_match} || '.*' ;
+            warn "class $config_class_name: name_match ($implicit$name_match)",
+                " in accept is deprecated\n" ;
+        }
+        
         push @accept_list, $name_match ;
-        $accept_hash{$name_match} = $accept_item ;
+        $accept_hash{$name_match} = shift @$accept_info;
     }
+
     $model->{accept}  = \%accept_hash ;
     $model->{accept_list}  = \@accept_list ;
 
@@ -1023,7 +1036,7 @@ sub check_class_parameters {
         (
          error => "create class $config_class_name: unexpected "
                 . "parameters '". join (', ', keys %$raw_model) ."' "
-                . "Expected '".join("', '",@legal_params)."'"
+                . "Expected '".join("', '",@legal_params, @static_legal_params)."'"
         )
           if keys %$raw_model ;
 
@@ -1434,10 +1447,13 @@ sub translate_follow_arg {
         map { $follow->{'f' . $idx++ } = $_ } @$raw_follow ;
         return $follow ;
     }
-    else {
+    elsif (defined $raw_follow) {
         # follow is a simple string
         return { f1 => $raw_follow } ;
     }
+    else {
+        return {} ;
+    }
 }
 
 sub translate_rules_arg {
@@ -1445,6 +1461,7 @@ sub translate_rules_arg {
         $raw_rules) = @_ ;
 
     my $multi_follow =  @$warper_items > 1 ? 1 : 0;
+    my $follow = @$warper_items ;
 
     # $rules is either:
     # { f1 => { ... } }  (  may be [ f1 => { ... } ] ?? )
@@ -1457,7 +1474,7 @@ sub translate_rules_arg {
         # transform the simple hash { foo => { ...} }
         # into array ref [ '$f1 eq foo' => { ... } ]
         my $h = $raw_rules ;
-        @rules = map { ( "\$f1 eq '$_'" , $h->{$_} ) } keys %$h ;
+        @rules = $follow ? map { ( "\$f1 eq '$_'" , $h->{$_} ) } keys %$h : keys %$h;
     }
     elsif (ref($raw_rules) eq 'ARRAY') {
         if ( $multi_follow ) {
@@ -1473,7 +1490,7 @@ sub translate_rules_arg {
             for (my $r_idx = 0; $r_idx < $#raw_rules; $r_idx  += 2) {
                 my $key_set = $raw_rules[$r_idx] ;
                 my @keys = ref($key_set) ? @$key_set : ($key_set) ;
-                my @bool_expr = map { /\$/ ? $_ : "\$f1 eq '$_'" } @keys ;
+                my @bool_expr = $follow ? map { /\$/ ? $_ : "\$f1 eq '$_'" } @keys : @keys ;
                 push @rules , join ( ' or ', @bool_expr),  $raw_rules[$r_idx+1] ;
             }
         }
@@ -1647,7 +1664,7 @@ sub include_one_class {
                 $raw_model->{$included_item} = $to_copy ;
             }
         }
-        else {
+        elsif ( not grep { $_ eq $included_item; } @static_legal_params ) {
             Config::Model::Exception::ModelDeclaration->throw
                 (
                  error => "Cannot include '$included_item', "
@@ -1711,9 +1728,9 @@ can also be declared within the element declaration:
   ) ;
 
 
-=head1 Load pre-declared model
+=head1 Load predeclared model
 
-You can also load pre-declared model.
+You can also load predeclared model.
 
 =head2 load( <model_name> )
 
@@ -1863,7 +1880,10 @@ Then, the merged model will feature C<fs_vfstype> with choice C<ext2 ext4 ext4>.
 Likewise, C<fs_mntopts> will feature rules for the 3 filesystems. 
 
 
-=head2 augment_config_class (name => '...', ... )
+=head2 augment_config_class (name => '...', class_data )
+
+Enhance the feature of a configuration class. This method uses the same parameters
+as L<create_config_class>.
 
 =cut
 
@@ -1904,7 +1924,8 @@ sub augment_config_class {
 
 =head2 get_model( config_class_name )
 
-Return a hash containing the model declaration.
+Return a hash containing the model declaration (in a deep clone copy of the hash).
+You may modify the hash at leisure.
 
 =cut
 
@@ -1914,7 +1935,7 @@ sub get_model {
       || die "Model::get_model: missing config class name argument" ;
 
     $self->load($config_class_name)
-      unless defined $self->model($config_class_name) ;
+      unless $self->model_exists($config_class_name) ;
 
     my $model = $self->model($config_class_name) ||
       croak "get_model error: unknown config class name: $config_class_name";
@@ -1922,6 +1943,178 @@ sub get_model {
     return dclone($model) ;
 }
 
+=head2 get_model_doc 
+
+Generate POD document for configuration class.
+
+=cut
+
+sub get_model_doc {
+    my ( $self, $top_class_name ) = @_;
+
+    if ( not defined $self->model($top_class_name) ) {
+        croak
+          "get_model_doc error : unknown config class name: $top_class_name";
+    }
+
+    my @classes = ($top_class_name);
+    my %result;
+
+    while (@classes) {
+        my $class_name = shift @classes;
+        next if defined $result{$class_name};
+        my $c_model   = $self->get_model($class_name)
+          || croak "get_model_doc model error : unknown config class name: $class_name";
+
+        my $full_name = "Config::Model::models::$class_name" ;
+
+        my %see_also ;
+
+        my @pod = (
+            "=head1 NAME",                                    '',
+            "$full_name - Configuration class " . $class_name,                    '',
+            "=head1 DESCRIPTION",                             '',
+            "Configuration classes used by L<Config::Model>", ''
+        );
+
+        my %legalese;
+
+        my $i = 0;
+
+        my $class_desc = $c_model->{class_description};
+        push @pod, $class_desc, '' if defined $class_desc;
+
+        my @elt = ( "=head1 Elements", '' );
+        foreach my $elt_name ( @{ $c_model->{element_list} } ) {
+            my $elt_info = $c_model->{element}{$elt_name};
+            my $summary = $elt_info->{summary} || '';
+            $summary &&= " - $summary" ;
+            push @elt, "=head2 $elt_name$summary", '';
+            push @elt, $self->get_element_description($elt_info) , '' ;
+
+            foreach ($elt_info,$elt_info->{cargo}) { 
+                my $ccn = $_->{config_class_name};
+                next unless defined $ccn ;
+                push @classes, $ccn ;
+                $see_also{$ccn} = 1;
+            }
+        }
+
+        foreach my $what (qw/author copyright license/) {
+            my $item = $c_model->{$what};
+            push @{ $legalese{$what} }, $item if $item;
+        }
+
+        my @end;
+        foreach my $what (qw/author copyright license/) {
+            next unless @{ $legalese{$what} || [] };
+            push @end, "=head1 " . uc($what), '', '=over', '',
+              ( map { ( "=item $_", '' ); } map {ref $_ ? @$_ : $_ } @{ $legalese{$what} } ),
+              '', '=back', '';
+        }
+
+        my @see_also =  (
+            "=head1 SEE ALSO",'',"=over",'',"=item *",'',"L<config-edit>",'',
+            ( map { ( "=item *",'',"L<Config::Model::models::$_>",'') ; } sort keys %see_also ),
+            "=back",'') ;
+
+        $result{$full_name} = join( "\n", @pod, @elt, @see_also, @end,'=cut','' ) . "\n";
+    }
+    return \%result ;
+}
+
+sub get_element_description {
+    my ( $self, $elt_info ) = @_;
+
+    my $type  = $elt_info->{type};
+    my $cargo = $elt_info->{cargo};
+    my $vt    = $elt_info->{value_type} ;
+
+    my $of         = '';
+    my $cargo_type = $cargo->{type};
+    my $cargo_vt   = $cargo->{value_type};
+    $of = " of " . ( $cargo_vt or $cargo_type ) if defined $cargo_type;
+
+    my $desc = $elt_info->{description} || '';
+    if ($desc) {
+        $desc .= '. ' if $desc =~ /\w$/ ;
+    }
+    
+    my $info = $elt_info->{mandatory} ? 'Mandatory. ' : 'Optional. ' ;
+
+    $info .= "Type ". ($vt || $type) . $of.'. ';
+    
+    foreach (qw/choice default upstream_default/) {
+        my $item = $elt_info->{$_} ;
+        next unless defined $item ;
+        my @list = ref($item) ? @$item : ($item) ;
+        $info .= "$_: '". join("', '",@list)."'. " ;
+    } 
+    
+    my $elt_help = $self->get_element_value_help ($elt_info) ;
+    
+    return $desc."I<< $info >>" .$elt_help;
+}
+
+sub get_element_value_help {
+    my ( $self, $elt_info ) = @_;
+
+    my $help = $elt_info->{help} ;
+    return '' unless defined $help ;
+    
+    my $help_text = "\n\nHere are some explanations on the possible values:\n\n=over\n\n" ;
+    foreach my $v (sort keys %$help) {
+        $help_text .= "=item $v\n\n$help->{$v}\n\n" ;
+    }
+    
+    return $help_text."=back\n\n" ;
+}
+
+=head2 generate_doc ( top_class_name , [ directory ] )
+
+Generate POD document for configuration class top_class_name 
+and write them on STDOUT or in specified directory.
+
+Returns a list of written file names.
+
+=cut
+
+sub generate_doc {
+    my ( $self, $top_class_name, $dir ) = @_;
+
+    my $res  = $self->get_model_doc($top_class_name) ;
+    my @wrote ;
+
+    if (defined $dir and $dir) {
+        foreach my $class_name (keys %$res) {
+            my $file = $class_name;
+            $file =~ s!::!/!g ;
+            my $pl_file   = $dir."/$file.pl";
+            my $pod_file  = $dir."/$file.pod";
+            my $pod_dir = $pod_file ;
+            $pod_dir =~ s!/[^/]+$!!;
+            make_path($pod_dir,{ mode => 0755} ) unless -d $pod_dir ;
+            # -M returns age of file, not date or epoch. hence greater is older
+            if (not -e $pod_file or not -e $pl_file or (-M $pl_file < -M $pod_file) ) {
+                #print "-M $pl_file: ", -M $pl_file, "-M $pod_file: ". -M $pod_file ,"\n";
+                my $fh = IO::File->new($pod_file,'>') || die "Can't open $pod_file: $!";
+                $fh->binmode(":utf8");
+                $fh->print($res->{$class_name});
+                $fh->close ;
+                print "Wrote documentation in $pod_file\n";
+                push @wrote, $pod_file ;
+            }
+        }
+    }
+    else {
+        foreach my $class_name (keys %$res) {
+            print "########## $class_name ############ \n\n";
+            print $res->{$class_name} ;
+        }
+    }
+    return @wrote ;
+}
+
 =head2 get_element_model( config_class_name , element)
 
 Return a hash containing the model declaration for the specified class
@@ -2021,7 +2214,25 @@ sub get_element_with_experience {
     return @result ;
 }
 
-#internal
+=head2 get_element_property
+
+Returns the property of an element from the model.
+
+Parameters are:
+
+=over 
+
+=item class 
+
+=item element 
+
+=item property
+
+=back 
+
+=cut
+
+
 sub get_element_property {
     my $self = shift ;
     my %args = @_ ;
@@ -2112,14 +2323,14 @@ Returns an array of 3 hash refs:
 category (system or user or application) => application list. E.g. 
 
  { system => [ 'popcon' , 'fstab'] }
- 
- =item *
+
+=item *
 
 application => { model => 'model_name', ... }
 
 =item *
 
-applicaiton => model_name
+application => model_name
 
 =back
 
@@ -2134,6 +2345,11 @@ sub available_models {
     get_logger("Model")->trace("available_models: path is $path");
     foreach my $dir (glob("$path/*.d")) {
         my ($cat) = ( $dir =~ m!.*/([\w\-]+)\.d! );
+
+        if ($cat !~ /^user|system|application$/) {
+            warn "available_models: skipping unexpected category: $cat\n";
+            next;
+        }
         
         get_logger("Model")->trace("available_models: category dir $dir");
         
@@ -2158,7 +2374,7 @@ sub available_models {
     return \%categories, \%appli_info, \%applications ;
 }
 
-no Moose ;
+no Any::Moose ;
 __PACKAGE__->meta->make_immutable ;
 
 1;
@@ -2169,35 +2385,29 @@ Errors are handled with an exception mechanism (See
 L<Exception::Class>).
 
 When a strongly typed Value object gets an authorized value, it raises
-an exception. If this exception is not catched, the programs exits.
+an exception. If this exception is not caught, the programs exits.
 
 See L<Config::Model::Exception|Config::Model::Exception> for details on
 the various exception classes provided with C<Config::Model>.
 
-=head1 Log and Traces
+=head1 Logging
 
-Currently a rather lame trace mechanism is provided:
-
-=over
-
-=item *
-
-Set C<$::debug> to 1 to get debug messages on STDOUT.
-
-=item *
-
-Set C<$::verbose> to 1 to get verbose messages on STDOUT.
-
-=back
-
-Depending on available time, a better log/error system may be
-implemented.
+See L<config-edit/Logging>
 
 =head1 BUGS
 
-Given Murphy's law, the author is fairly confident that you will find bugs or miss some features. Please report thems 
-config-model at rt.cpan.org, or through the web interface at https://rt.cpan.org/Public/Bug/Report.html?Queue=config-model .
-The author will be notified, and then you'll automatically be notified of progress on your bug.
+Given Murphy's law, the author is fairly confident that you will find
+bugs or miss some features. Please report them to config-model at
+rt.cpan.org, or through the web interface at 
+https://rt.cpan.org/Public/Bug/Report.html?Queue=config-model . 
+The author will be notified, and then you'll automatically be
+notified of progress on your bug.
+
+=head1 FEEDBACK
+
+Feedback from users are highly desired. If you find this module useful, please
+share your use cases, success stories with the author or with the config-model-
+users mailing list. 
 
 =head1 AUTHOR
 
@@ -2205,12 +2415,12 @@ Dominique Dumont, (ddumont at cpan dot org)
 
 =head1 LICENSE
 
-    Copyright (c) 2005-2010 Dominique Dumont.
+    Copyright (c) 2005-2011 Dominique Dumont.
 
     This file is part of Config-Model.
 
     Config-Model is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Lesser Public License as
+    modify it under the terms of the GNU Lesser General Public License as
     published by the Free Software Foundation; either version 2.1 of
     the License, or (at your option) any later version.
 
@@ -2219,7 +2429,7 @@ Dominique Dumont, (ddumont at cpan dot org)
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
     Lesser Public License for more details.
 
-    You should have received a copy of the GNU Lesser Public License
+    You should have received a copy of the GNU Lesser General Public License
     along with Config-Model; if not, write to the Free Software
     Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
     02110-1301 USA
@@ -2238,28 +2448,27 @@ The arrow shows the inheritance of the classes
 
 =item *
 
-L<Config::Model::Node> <- L<Config::Model::AutoRead> <- L<Config::Model::AnyThing>
+L<Config::Model::Node> <- L<Config::Model::AnyThing>
 
 =item *
 
-L<Config::Model::HashId> <- L<Config::Model::AnyId> <- L<Config::Model::WarpedThing> <- L<Config::Model::AnyThing>
+L<Config::Model::HashId> <- L<Config::Model::AnyId> <- L<Config::Model::AnyThing>
 
 =item *
 
-L<Config::Model::ListId> <- L<Config::Model::AnyId> <- L<Config::Model::WarpedThing> <- L<Config::Model::AnyThing>
+L<Config::Model::ListId> <- L<Config::Model::AnyId> <- L<Config::Model::AnyThing>
 
 =item *
 
-L<Config::Model::Value> <- L<Config::Model::WarpedThing> <- L<Config::Model::AnyThing>
+L<Config::Model::Value> <- L<Config::Model::AnyThing>
 
 =item *
 
-L<Config::Model::CheckList> <- L<Config::Model::WarpedThing> <- L<Config::Model::AnyThing>
+L<Config::Model::CheckList> <- L<Config::Model::AnyThing>
 
 =item *
 
-L<Config::Model::WarpedNode> <- L<Config::Model::WarpedThing> <- L<Config::Model::AnyThing>
-
+L<Config::Model::WarpedNode> <- L<Config::Model::AnyThing>
 
 =back
 
@@ -2273,10 +2482,18 @@ L<config-edit>
 
 =item *
 
+L<Config::Model::Backend::Fstab> <- L<Config::Model::Backend::Any>
+
+=item *
+
 L<Config::Model::Backend::IniFile> <- L<Config::Model::Backend::Any>
 
 =item *
 
+L<Config::Model::Backend::PlainFile> <- L<Config::Model::Backend::Any>
+
+=item *
+
 L<Config::Model::Backend::ShellVar> <- L<Config::Model::Backend::Any>
 
 =item *
@@ -2295,6 +2512,10 @@ L<Config::Model::Annotation>
 
 =item *
 
+L<Config::Model::BackendMgr>: Used by C<Config::Model::Node> object
+
+=item *
+
 L<Config::Model::Describe>
 
 =item *
@@ -2307,6 +2528,10 @@ L<Config::Model::DumpAsData>
 
 =item *
 
+L<Config::Model::IdElementReference>
+
+=item *
+
 L<Config::Model::Loader>
 
 =item *
@@ -2319,7 +2544,15 @@ L<Config::Model::Report>
 
 =item *
 
-L<Config::Model::Searcher>
+L<Config::Model::Searcher>: Search element in configuration model.
+
+=item *
+
+L<Config::Model::SimpleUI>
+
+=item *
+
+L<Config::Model::TreeSearcher>: Search string or regexp in configuration tree.
 
 =item *
 
@@ -2327,7 +2560,7 @@ L<Config::Model::TermUI>
 
 =item *
 
-L<Config::Model::WizardHelper>
+L<Config::Model::Iterator>
 
 =item *
 
@@ -2337,6 +2570,10 @@ L<Config::Model::AutoRead>
 
 L<Config::Model::ValueComputer>
 
+=item *
+
+L<Config::Model::Warper>
+
 =back
 
-=cut
\ No newline at end of file
+=cut
