@@ -1,13 +1,13 @@
 #
 # This file is part of Config-Model
 #
-# This software is Copyright (c) 2010 by Dominique Dumont, Krzysztof Tyszecki.
+# This software is Copyright (c) 2011 by Dominique Dumont, Krzysztof Tyszecki.
 #
 # This is free software, licensed under:
 #
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
-#    Copyright (c) 2005-2010 Dominique Dumont.
+#    Copyright (c) 2005-2011 Dominique Dumont.
 #
 #    This file is part of Config-Model.
 #
@@ -26,17 +26,19 @@
 #    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 
 package Config::Model::AnyId ;
-BEGIN {
-  $Config::Model::AnyId::VERSION = '1.226';
+{
+  $Config::Model::AnyId::VERSION = '1.260';
 }
 use Config::Model::Exception ;
+use Config::Model::Warper ;
 use Scalar::Util qw(weaken) ;
 use warnings ;
-use Carp;
+use Carp qw/cluck croak carp/;
 use strict;
 use Log::Log4perl qw(get_logger :levels);
+use Storable qw/dclone/;
 
-use base qw/Config::Model::WarpedThing/;
+use base qw/Config::Model::AnyThing/;
 
 my $logger = get_logger("Tree::Element::Id") ;
 
@@ -54,44 +56,83 @@ Config::Model::AnyId - Base class for hash or list element
 
 =head1 VERSION
 
-version 1.226
+version 1.260
 
 =head1 SYNOPSIS
 
- $model ->create_config_class 
-  (
-   ...
-   element 
-   => [ 
-       bounded_hash 
-       => { type => 'hash',                 # hash id
-            index_type  => 'integer',
+ use Config::Model;
+ use Log::Log4perl qw(:easy);
+ Log::Log4perl->easy_init($WARN);
+
+ # define configuration tree object
+ my $model = Config::Model->new;
+ $model->create_config_class(
+    name    => "Foo",
+    element => [
+        [qw/foo bar/] => {
+            type       => 'leaf',
+            value_type => 'string'
+        },
+    ]
+ );
+
+ $model->create_config_class(
+    name    => "MyClass",
+    element => [
+        plain_hash => {
+            type       => 'hash',
+            index_type => 'string',
+            cargo      => {
+                type       => 'leaf',
+                value_type => 'string',
+            },
+        },
+        bounded_hash => {
+            type       => 'hash',      # hash id
+            index_type => 'integer',
 
             # hash boundaries
-            min_index => 1, max_index => 123, max_nb => 2 ,
+            min_index => 1, max_index => 123, max_nb => 2,
 
             # specify cargo held by hash
-            cargo => { type => 'leaf',
-                       value_type => 'string'
-                     },
-          },
-      bounded_list 
-       => { type => 'list',                 # list id
+            cargo => {
+                type       => 'leaf',
+                value_type => 'string'
+            },
+        },
+        bounded_list => {
+            type => 'list',    # list id
 
-            max_index => 123, 
-            cargo => { type => 'leaf',
-                       value_type => 'string'
-                     },
-          },
-      hash_of_nodes 
-      => { type => 'hash',                 # hash id
-           index_type  => 'integer',
-           cargo => { type => 'node',
-                      config_class_name => 'Foo'
-                    },
-         },
-      ]
-  ) ;
+            max_index => 123,
+            cargo     => {
+                type       => 'leaf',
+                value_type => 'string'
+            },
+        },
+        hash_of_nodes => {
+            type       => 'hash',     # hash id
+            index_type => 'string',
+            cargo      => {
+                type              => 'node',
+                config_class_name => 'Foo'
+            },
+        },
+    ],
+ );
+
+ my $inst = $model->instance( root_class_name => 'MyClass' );
+
+ my $root = $inst->config_root;
+
+ # put data
+ my $step = 'plain_hash:foo=boo bounded_list=foo,bar,baz
+   bounded_hash:3=foo bounded_hash:30=baz 
+   hash_of_nodes:"foo node" foo="in foo node" -
+   hash_of_nodes:"bar node" bar="in bar node" ';
+ $root->load( step => $step );
+
+ # dump resulting tree
+ print $root->dump_tree;
 
 =head1 DESCRIPTION
 
@@ -114,7 +155,10 @@ sub new {
     # args hash is modified for arg check in derived class constructor
     my $args_ref = shift ; 
 
-    my $self= { warning_hash => { } } ;
+    my $self= { 
+            warning_hash => { },
+            warning_list => [ ],
+        } ;
 
     bless $self,$type;
 
@@ -172,6 +216,15 @@ order can be modified with L<swap|Config::Model::HashId/"swap ( key1 , key2 )">,
 L<move_up|Config::Model::HashId/"move_up ( key )"> or
 L<move_down|Config::Model::HashId/"move_down ( key )">.
 
+=item duplicates
+
+Specify the policy regarding duplicated values stored in the list or as
+hash values (valid only when cargo type is C<leaf>). The policy can be
+C<allow> (default), C<suppress>, C<warn> (which offers the possibility
+to apply a fix), C<forbid>. Note that duplicates I<check cannot be
+performed when the duplicated value is stored>: this happens outside of
+this object. Duplicates can be check only after when the value is read.
+
 =item cargo
 
 Hash ref specifying the cargo held by the hash of list. This has must
@@ -212,8 +265,8 @@ be used with string index type)
 =item default_keys
 
 When set, the default parameter (or set of parameters) are used as
-default keys hashes and created automatically when the keys or exists
-functions are used on an I<empty> hash.
+default keys hashes and created automatically when the C<keys> or C<exists>
+functions are used on an I<empty> hash..
 
 You can use C<< default_keys => 'foo' >>, 
 or C<< default_keys => ['foo', 'bar'] >>.
@@ -222,9 +275,21 @@ or C<< default_keys => ['foo', 'bar'] >>.
 
 To perform special set-up on children nodes you can also use 
 
-   default_with_init =>  { 'foo' => 'X=Av Y=Bv'  ,
+   default_with_init =>  { 'foo' => 'X=Av Y=Bv' ,
                            'bar' => 'Y=Av Z=Cv' }
+                           
+When the hash contains leaves, you can also use:
 
+   default_with_init => { 'def_1' => 'def_1 stuff' ,
+                          'def_2' => 'def_2 stuff' }
+
+
+=item migrate_keys_from
+
+Specifies that the keys of the hash or list are copied from another hash or list in
+the configuration tree only when the hash is created. 
+
+   migrate_keys_from => '- another_hash_or_list'
 
 =item follow_keys_from
 
@@ -258,8 +323,8 @@ Keys must match the specified regular expression. For instance:
 When set, the default parameter (or set of parameters) are used as
 keys hashes and created automatically. (valid only for hash elements)
 
-Called with C<< auto_create => 'foo' >>, or 
-C<< auto_create => ['foo', 'bar'] >>.
+Called with C<< auto_create_keys => ['foo'] >>, or 
+C<< auto_create_keys => ['foo', 'bar'] >>.
 
 =item warn_if_key_match
 
@@ -272,7 +337,7 @@ Issue a warning unless the key matches the specified regular expression
 =item auto_create_ids
 
 Specifies the number of elements to create automatically. E.g.  C<<
-auto_create => 4 >> will initialize the list with 4 undef elements.
+auto_create_ids => 4 >> will initialize the list with 4 undef elements.
 (valid only for list elements)
 
 =item warp
@@ -344,9 +409,9 @@ auto created.
 For instance, if a tied hash is created with 
 C<< auto_create => [a,b,c] >>, the hash contains C<(a,b,c)>.
 
-Then if a warp is applied with C<< auto_create => [c,d,e] >>, the hash
+Then if a warp is applied with C<< auto_create_keys => [c,d,e] >>, the hash
 will contain C<(a,b,c,d,e)>. The items created by the first
-auto_create are not removed.
+auto_create_keys are not removed.
 
 =head2 Warp and max_nb
 
@@ -354,12 +419,13 @@ When a warp is applied, the items that do not fit the constraint
 (e.g. min_index, max_index) are removed.
 
 For the max_nb constraint, an exception will be raised if a warp 
-leads to a nb of items greater than the max_nb constraint.
+leads to a number of items greater than the max_nb constraint.
 
 =cut
 
 my @common_params =  qw/min_index max_index max_nb default_with_init default_keys
-                        follow_keys_from auto_create_ids auto_create_keys
+                        follow_keys_from migrate_keys_from duplicates
+                        auto_create_ids auto_create_keys
                         allow_keys allow_keys_from allow_keys_matching
                         warn_if_key_match warn_unless_key_match/ ;
 
@@ -376,6 +442,9 @@ sub set_properties {
 
     my %args = (%{$self->{backup}},@_) ;
 
+    # these are handled by Node or Warper
+    map { delete $args{$_} } qw/level experience/ ;
+
     $logger->debug($self->name," set_properties called with @_");
 
     map { $self->{$_} =  delete $args{$_} if defined $args{$_} }
@@ -400,7 +469,7 @@ sub set_properties {
         my $last_idx  = pop   @current_idx ;
 
         foreach my $idx ( ($first_idx, $last_idx)) {
-            my $ok = $self->check($first_idx) ;
+            my $ok = $self->check_idx($first_idx) ;
             next if $ok ;
 
             # here a user input may trigger an exception even if fetch
@@ -424,43 +493,78 @@ sub set_properties {
     if (defined $self->{auto_create_keys} or defined $self->{auto_create_ids}) {
         $self->auto_create_elements ;
     }
+    
+    if (    defined $self->{duplicates}
+        and defined $self->{cargo}
+        and $self->{cargo}{type} ne 'leaf' )
+    {
+        Config::Model::Exception::Model->throw(
+            object => $self,
+            error => "Cannot specify 'duplicates' with cargo type '$self->{cargo}{type}'",
+        );
+    }
 
-    $self->{current} = { level      => $args{level} ,
-                         experience => $args{experience}
-                       } ;
-    $self->SUPER::set_parent_element_property(\%args) ;
+    my $ok_dup = 'forbid|suppress|warn|allow';
+    if ( defined $self->{duplicates} and $self->{duplicates} !~ /^$ok_dup$/ ) {
+        Config::Model::Exception::Model->throw(
+            object => $self,
+            error => "Unexpected 'duplicates' $self->{duplicates} expected $ok_dup",
+        );
+    }
 
-    Config::Model::Exception::Model
-        ->throw (
+    # $self->{current} = { level      => $args{level} ,
+                         # experience => $args{experience}
+                       # } ;
+    #$self->SUPER::set_parent_element_property(\%args) ;
+    
+    Config::Model::Exception::Model ->throw (
                  object => $self,
                  error => "Unexpected parameters: ". join(' ', keys %args)
                 ) if scalar keys %args ;
 }
 
-# this method will overide setting comings from a value (or maybe a
-# warped node) with warped settings coming from this warped id. Only
-# level hidden is forwarded no matter what
-sub set_parent_element_property {
+# # this method will overide setting comings from a value (or maybe a
+# # warped node) with warped settings coming from this warped id. Only
+# # level hidden is forwarded no matter what
+# sub set_parent_element_property {
+    # my $self = shift;
+    # my $arg_ref = shift ;
+# 
+    # my $cur = $self->{current} ;
+# 
+    # # override if necessary
+    # $arg_ref->{experience} = $cur->{experience} 
+      # if defined $cur->{experience} ;
+# 
+    # if (    defined $cur->{level} 
+        # and ( not defined $arg_ref->{level} 
+              # or $arg_ref->{level} ne 'hidden'
+            # )
+       # ) {
+        # $arg_ref->{level} = $cur->{level} ;
+    # }
+# 
+    # $self->SUPER::set_parent_element_property($arg_ref) ;
+# }
+
+sub create_default_with_init {
     my $self = shift;
-    my $arg_ref = shift ;
 
-    my $cur = $self->{current} ;
+    return unless defined $self->{default_with_init};
 
-    # override if necessary
-    $arg_ref->{experience} = $cur->{experience} 
-      if defined $cur->{experience} ;
-
-    if (    defined $cur->{level} 
-        and ( not defined $arg_ref->{level} 
-              or $arg_ref->{level} ne 'hidden'
-            )
-       ) {
-        $arg_ref->{level} = $cur->{level} ;
+    my $h = $self->{default_with_init};
+    foreach my $def_key ( keys %$h ) {
+        my $v_obj = $self->fetch_with_id($def_key);
+        if ( $v_obj->get_type eq 'leaf' ) {
+            $v_obj->store( $h->{$def_key} );
+        }
+        else {
+            $v_obj->load( $h->{$def_key} );
+        }
     }
-
-    $self->SUPER::set_parent_element_property($arg_ref) ;
 }
 
+
 =head1 Introspection methods
 
 The following methods returns the current value stored in the Id
@@ -555,11 +659,13 @@ sub get_cargo_info {
 # internal, does a grab with improved error mesage
 sub safe_typed_grab {
   my $self  = shift ;
-  my $param = shift ;
+  my %args = @_ ;
+  my $param = $args{param} || croak "safe_typed_grab: missing param" ;
 
   my $res = eval {
     $self->grab(step => $self->{$param},
                 type => $self->get_type,
+                check => $args{check} || 'yes' ,
                ) ;
   };
 
@@ -589,12 +695,18 @@ sub get_default_keys {
     my $self = shift ;
 
     if ($self->{follow_keys_from}) {
-        my $followed = $self->safe_typed_grab('follow_keys_from') ;
-        return [ $followed -> get_all_indexes ];
+        my $followed = $self->safe_typed_grab(param => 'follow_keys_from') ;
+        my @res = $followed -> get_all_indexes ;
+        return wantarray ? @res : \@res ;
     }
 
     my @res ;
 
+    if ($self->{migrate_keys_from}) {
+        my $followed = $self->safe_typed_grab(param => 'migrate_keys_from', check => 'no') ;
+        push @res , $followed -> get_all_indexes ;
+    }
+
     push @res , @{ $self->{default_keys} }
       if defined $self->{default_keys} ;
 
@@ -641,32 +753,67 @@ sub handle_args {
     map { $self->{$_} =  delete $args{$_} if defined $args{$_} }
          qw/index_class index_type morph ordered/;
 
-    %{$self->{backup}}  = %args ;
+    $self->{backup}  = dclone (\%args) ;
 
     $self->set_properties(%args) if defined $self->{index_type} ;
 
     if (defined $warp_info) {
-        $self->check_warp_args( \@allowed_warp_params, $warp_info) ;
+        $self->{warper} = Config::Model::Warper->new (
+            warped_object => $self,
+            %$warp_info ,
+            allowed => \@allowed_warp_params
+        ) ;
     }
 
-    $self->submit_to_warp($self->{warp}) if $self->{warp} ;
-
     return $self ;
 }
 
-my %check_dispatch = map { ($_ => 'check_'.$_) ;}
-    qw/follow_keys_from allow_keys allow_keys_from allow_keys_matching
-        warn_if_key_match warn_unless_key_match/;
+sub apply_fixes {
+    my $self = shift ; 
+    $logger->debug( $self->location.": apply_fixes called" ) ;
 
-# internal function to check the validity of the index
+    $self->check(fix => 1) ;
+
+}
+
+=head2 has_fixes
+
+Returns the number of fixes that can be applied to the current value. 
+
+=cut
+
+sub has_fixes {
+    my $self = shift; 
+    return $self->{nb_of_fixes} ;
+}
+
+
+
+my %check_idx_dispatch =
+  map { ( $_ => 'check_' . $_ ); }
+  qw/follow_keys_from allow_keys allow_keys_from allow_keys_matching
+  warn_if_key_match warn_unless_key_match/;
+
+my %check_dispatch = map { ($_ => 'check_'.$_) ;}
+    qw/duplicates/;
+
+# check globally the list or hash
 sub check {
-    my ($self,$idx) = @_ ; 
+    my $self = shift ;
+    
+    my %args = @_ > 1 ? @_ : (index => $_[0]) ;
+    my $silent = $args{silent} || 0 ;
+    my $check = $args{check} || 'yes' ;
+    my $apply_fix = $args{fix} || 0 ;
+
+    # need to keep track to update GUI
+    $self->{nb_of_fixes} = 0; # reset before check
 
     Config::Model::Exception::Internal
         -> throw (
                   object => $self,
-                  error => "check method: key or index is not defined"
-                 ) unless defined $idx ;
+                  error => "check method: index or key should not be defined"
+                 ) if defined $args{index} ;
 
     my @error ;
     my @warn ;
@@ -674,7 +821,43 @@ sub check {
     foreach my $key_check_name (keys %check_dispatch) {
         next unless $self->{$key_check_name} ;
         my $method = $check_dispatch{$key_check_name} ;
-        $self->$method($idx,\@error,\@warn) ;
+        $self->$method(\@error,\@warn,$apply_fix) ;
+    }
+
+    my $nb =  $self->fetch_size ;
+    push @error,"Too many instances ($nb) limit $self->{max_nb}, "
+        if defined $self->{max_nb} and $nb > $self->{max_nb};
+
+    map { warn ("Warning in '".$self->location."': $_\n") } @warn unless $silent;
+    
+    $self->{warning_list} = \@warn ;
+
+    return scalar @error ? 0 : 1 ;
+}
+
+# internal function to check the validity of the index
+sub check_idx {
+    my $self = shift ;
+    
+    my %args = @_ > 1 ? @_ : (index => $_[0]) ;
+    my $idx = $args{index} ;
+    my $silent = $args{silent} || 0 ;
+    my $check = $args{check} || 'yes' ;
+    my $apply_fix = $check eq 'fix' ? 1 : 0 ;
+
+    Config::Model::Exception::Internal
+        -> throw (
+                  object => $self,
+                  error => "check_idx method: key or index is not defined"
+                 ) unless defined $idx ;
+
+    my @error ;
+    my @warn ;
+
+    foreach my $key_check_name (keys %check_idx_dispatch) {
+        next unless $self->{$key_check_name} ;
+        my $method = $check_idx_dispatch{$key_check_name} ;
+        $self->$method($idx,\@error,\@warn,$apply_fix) ;
     }
 
     my $nb =  $self->fetch_size ;
@@ -708,19 +891,20 @@ sub check {
 
     if (@warn) {
         $self->{warning_hash}{$idx} = \@warn ;
-        warn(map { "Warning in '".$self->location."': $_\n"} @warn) ;
+        map { warn ("Warning in '".$self->location."': $_\n") } @warn unless $silent;
+    }
+    else {
+        delete $self->{warning_hash}{$idx} ;
     }
         
     return scalar @error ? 0 : 1 ;
 }
 
-
-
 #internal
 sub check_follow_keys_from {
     my ($self,$idx,$error) = @_ ; 
 
-    my $followed = $self->safe_typed_grab('follow_keys_from') ;
+    my $followed = $self->safe_typed_grab(param => 'follow_keys_from') ;
     return if $followed->exists($idx) ;
 
     push @$error, "key '$idx' does not exists in '".$followed->name 
@@ -751,7 +935,7 @@ sub check_allow_keys_matching {
 sub check_allow_keys_from {
     my ($self,$idx,$error) = @_ ; 
 
-    my $from = $self->safe_typed_grab('allow_keys_from');
+    my $from = $self->safe_typed_grab(param => 'allow_keys_from');
     my $ok = grep { $_ eq $idx } $from->get_all_indexes ;
 
     return if $ok ;
@@ -777,12 +961,60 @@ sub check_warn_unless_key_match {
     push @$warn, "key '$idx' should match $re\n" unless $idx =~ /$re/;
 }
 
-=head1 Informations management
+sub check_duplicates {
+    my ( $self, $error, $warn, $apply_fix ) = @_;
+
+    my $dup = $self->{duplicates} ;
+    return if $dup eq 'allow' ;
+    
+    $logger->debug("check_duplicates called");
+    my %h;
+    my @issues;
+    my @to_delete ;
+    foreach my $i ( $self->get_all_indexes ) {
+        my $v = $self->fetch_with_id(index => $i, check => 'no')->fetch;
+        next unless defined $v ;
+        $h{$v} = 0 unless defined $h{$v} ;
+        $h{$v}++;
+        if ($h{$v} > 1) {
+            $logger->debug("got duplicates $i -> $v : $h{$v}");
+            push @to_delete, $i ;
+            push @issues, qq!$i:"$v"! ; 
+        }
+    }
+
+    return unless @issues ;
+    
+    if ($apply_fix) {
+        $logger->debug("Fixing duplicates @issues");
+        map { $self->remove($_) } reverse @to_delete ;
+    }
+    elsif ($dup eq 'forbid') {
+        $logger->debug("Found forbidden duplicates @issues");
+        push @$error, "Forbidden duplicates value @issues";
+    }
+    elsif ($dup eq 'warn') {
+        $logger->debug("warning condition: found duplicate @issues");
+        push @$warn, "Duplicated value: @issues";
+        $self->{nb_of_fixes} += scalar @issues ;
+    }
+    elsif ($dup eq 'suppress') {
+        $logger->debug("suppressing duplicates @issues");
+        map { $self->remove($_) } reverse @to_delete ;
+    }
+    else {
+        die "Internal error: duplicates is $dup";
+    }
+}
+
+
+
+=head1 Information management
 
 =head2 fetch_with_id ( index => $idx , [ check => 'no' ])
 
 Fetch the collected element held by the hash or list. Index check is 'yes' by default.
-Can be called with one parameter: idx.
+Can be called with one parameter which will be used as index.
 
 =cut
 
@@ -795,7 +1027,7 @@ sub fetch_with_id {
     $self->warp 
       if ($self->{warp} and @{$self->{warp_info}{computed_master}});
 
-    my $ok = $self->check($idx) ;
+    my $ok = $check eq 'no' ? 1 : $self->check_idx(index => $idx, check => $check) ;
 
     if ($ok or $check eq 'no') {
         $self->auto_vivify($idx) unless $self->_defined($idx) ;
@@ -812,7 +1044,7 @@ sub fetch_with_id {
     return ;
 }
 
-=head2 get( path => ..., mode => ... ,  check => ... , get_obj => 1|0, autoadd => 1|0)
+=head2 C<get( path => ..., mode => ... ,  check => ... , get_obj => 1|0, autoadd => 1|0)>
 
 Get a value from a directory like path.
 
@@ -864,7 +1096,7 @@ sub copy {
     my ($self,$from, $to) = @_ ;
 
     my $from_obj = $self->fetch_with_id($from) ;
-    my $ok = $self->check($to) ;
+    my $ok = $self->check_idx($to) ;
 
     if ($ok && $self->{cargo}{type} eq 'leaf') {
         $logger->trace("AnyId: copy leaf value from ".$self->name." $from to $to") ;
@@ -936,24 +1168,37 @@ sub fetch_all_values {
     
     my @keys  = $self->get_all_indexes ;
 
-    if ($self->{cargo}{type} eq 'leaf') {
-        return grep {defined $_} 
-          map { $self->fetch_with_id($_)->fetch(check => $check, mode => $mode) ;} @keys ;
+    if ( $self->{cargo}{type} eq 'leaf' ) {
+        my $ok = $check eq 'no' ? 1 : $self->check( check => $check );
+
+        if ( $ok or $check eq 'no' ) {
+            return grep { defined $_ }
+              map {
+                $self->fetch_with_id($_)
+                  ->fetch( check => $check, mode => $mode );
+              } @keys;
+        }
+        else {
+            Config::Model::Exception::WrongValue->throw(
+                error  => join( "\n\t", @{ $self->{error} } ),
+                object => $self
+            );
+        }
+
     }
     else {
-        my $info = "current keys are '".join("', '",@keys)."'." ;
-        if ($self->{cargo}{type} eq 'node') {
-            $info .= "config class is ".
-              $self->fetch_with_id($keys[0])->config_class_name ;
+        my $info = "current keys are '" . join( "', '", @keys ) . "'.";
+        if ( $self->{cargo}{type} eq 'node' ) {
+            $info .= "config class is "
+              . $self->fetch_with_id( $keys[0] )->config_class_name;
         }
-        Config::Model::Exception::WrongType
-            ->throw(
-                    object => $self,
-                    function => 'fetch_all_values',
-                    got_type => $self->{cargo}{type},
-                    expected_type => 'leaf',
-                    info => $info,
-                   )
+        Config::Model::Exception::WrongType->throw(
+            object        => $self,
+            function      => 'fetch_all_values',
+            got_type      => $self->{cargo}{type},
+            expected_type => 'leaf',
+            info          => $info,
+        );
     }
 }
 
@@ -966,9 +1211,7 @@ are sorted alphabetically, except for ordered hashed.
 
 sub get_all_indexes {
     my $self = shift;
-    $self->create_default if (   defined $self->{default_keys}
-                              or defined $self->{default_with_init}
-                              or defined $self->{follow_keys_from});
+    $self->create_default ; # will check itself if creation is necessary
     return $self->_get_all_indexes ;
 }
 
@@ -1008,8 +1251,8 @@ my %can_override_class
 #internal
 sub auto_vivify {
     my ($self,$idx) = @_ ;
-    my $class = $self->{cargo_class} ; # to override class in cargo
     my %cargo_args = %{$self->{cargo} || {}} ;
+    my $class = delete $cargo_args{class} ; # to override class in cargo
 
     my $cargo_type = delete $cargo_args{type} ;
 
@@ -1165,15 +1408,35 @@ sub clear_values {
 =head2 warning_msg ( [index] )
 
 Returns warnings concerning indexes of this hash. 
-Without parameter, returns a hash ref or undef. With an index, return the warnings
+Without parameter, returns a string containing all warnings or undef. With an index, return the warnings
 concerning this index or undef.
 
 =cut
 
 sub warning_msg {
     my ($self,$idx) = @_ ;
-    return unless scalar %{$self->{warning_hash}};
-    return defined $idx ? $self->{warning_hash}{$idx} : $self->{warning_hash} ;
+    
+    if (defined $idx) {
+        return $self->{warning_hash}{$idx} ;
+    }
+    elsif (scalar %{$self->{warning_hash}} or @{$self->{warning_list}}) {
+        my @list = @{$self->{warning_list}} ;
+        push @list , map ( "key $_: ".$self->{warning_hash}{$_}, keys %{$self->{warning_hash}}) ;
+        return join("\n",@list) ;
+    }
+}
+
+
+=head2 error_msg 
+
+Returns the error messages of this object (if any)
+
+=cut
+
+sub error_msg {
+    my $self = shift ;
+    return unless $self->{error} ;
+    return wantarray ? @{$self->{error}} : join("\n\t",@{ $self ->{error}}) ;
 }
 
 1;
@@ -1195,4 +1458,4 @@ L<Config::Model::ListId>,
 L<Config::Model::CheckList>,
 L<Config::Model::Value>
 
-=cut
\ No newline at end of file
+=cut
