@@ -1,7 +1,7 @@
 #
 # This file is part of Config-Model
 #
-# This software is Copyright (c) 2010 by Dominique Dumont, Krzysztof Tyszecki.
+# This software is Copyright (c) 2011 by Dominique Dumont, Krzysztof Tyszecki.
 #
 # This is free software, licensed under:
 #
@@ -27,8 +27,8 @@
 #    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 
 package Config::Model::Dumper;
-BEGIN {
-  $Config::Model::Dumper::VERSION = '1.226';
+{
+  $Config::Model::Dumper::VERSION = '1.260';
 }
 use Carp;
 use strict;
@@ -43,39 +43,58 @@ Config::Model::Dumper - Serialize data of config tree
 
 =head1 VERSION
 
-version 1.226
+version 1.260
 
 =head1 SYNOPSIS
 
  use Config::Model ;
+ use Log::Log4perl qw(:easy) ;
+ Log::Log4perl->easy_init($WARN);
 
- # create your config model
- my $model = Config::Model -> new ;
- $model->create_config_class( ... ) ;
+ # define configuration tree object
+ my $model = Config::Model->new ;
+ $model ->create_config_class (
+    name => "MyClass",
+    element => [ 
+        [qw/foo bar/] => { 
+            type => 'leaf',
+            value_type => 'string'
+        },
+        baz => { 
+            type => 'hash',
+            index_type => 'string' ,
+            cargo => {
+                type => 'leaf',
+                value_type => 'string',
+            },
+        },
+        
+    ],
+ ) ;
 
- # create instance
- my $inst = $model->instance (root_class_name => 'FooBar', 
-			      instance_name => 'test1');
+ my $inst = $model->instance(root_class_name => 'MyClass' );
 
- # create root of config
- my $root = $inst -> config_root ;
+ my $root = $inst->config_root ;
 
- # put some data in config tree
- my $step = 'std_id:ab X=Bv - std_id:bc X=Av - a_string="toto tata"';
- $root->walk( step => $step ) ;
+ # put some data in config tree the hard way
+ $root->fetch_element('foo')->store('yada') ;
+ $root->fetch_element('bar')->store('bla bla') ;
+ $root->fetch_element('baz')->fetch_with_id('en')->store('hello') ;
 
- # dump only customized data (audit mode)
+ # put more data the easy way
+ my $step = 'baz:fr=bonjour baz:hr="dobar dan"';
+ $root->load( step => $step ) ;
+
+ # dump only customized data
  print $root->dump_tree;
 
- # dump all data including default values
- print $root->dump_tree( full_dump => 1 ) ;
 
 =head1 DESCRIPTION
 
 This module is used directly by L<Config::Model::Node> to serialize 
 configuration data in a compact (but readable) string.
 
-The serialisation can be done in standard mode where only customized
+The serialization can be done in standard mode where only customized
 values are dumped in the string. I.e. only data modified by the user
 are dumped.
 
@@ -104,9 +123,18 @@ sub new {
 sub quote {
     my @res = @_ ;
     foreach (@res) {
-	if (    defined $_ 
-		and ( /(\s|"|\*)/ or $_ eq '')
-	   ) {
+	if ( defined $_ and ( /(\s|"|\*)/ or $_ eq '') ) {
+	    s/"/\\"/g ; # escape present quotes
+	    $_ = '"' . $_ . '"' ; # add my quotes
+	}
+    }
+    return wantarray ? @res : $res[0];
+}
+
+sub note_quote {
+    my @res = @_ ;
+    foreach (@res) {
+	if ( defined $_ and $_ and ( /(\s|"|\*)/ ) ) {
 	    s/"/\\"/g ; # escape present quotes
 	    $_ = '"' . $_ . '"' ; # add my quotes
 	}
@@ -218,7 +246,7 @@ sub dump_tree {
                  :                   $element;
 
 	# add annotation for obj contained in hash or list
-	my $note = quote($value_obj->annotation) ;
+	my $note = note_quote($value_obj->annotation) ;
         $$data_r .= "\n" . $pad . $name if defined $value or $note ;
 	$$data_r .= '='  . $value       if defined $value          ;
 	$$data_r .= '#'  . $note        if                   $note ;
@@ -237,7 +265,7 @@ sub dump_tree {
                  :                   $element;
 
 	# add annotation for obj contained in hash or list
-	my $note = quote($value_obj->annotation) ;
+	my $note = note_quote($value_obj->annotation) ;
         $$data_r .= "\n" . $pad . $name if $value or $note ;
 	$$data_r .= '='  . $qvalue      if $value          ;
 	$$data_r .= '#'  . $note        if           $note ;
@@ -250,7 +278,7 @@ sub dump_tree {
 	my $list_obj = $node->fetch_element($element) ;
 
 	# add annotation for list element
-	my $list_note  = quote($list_obj->annotation) ;
+	my $list_note  = note_quote($list_obj->annotation) ;
 	$$data_r .= "\n$pad$element#$list_note" if $list_note ;
 
         if ( $list_obj->cargo_type eq 'node' ) {
@@ -259,14 +287,16 @@ sub dump_tree {
             }
         }
         else {
+            # write value comments
+            foreach my $idx ($list_obj->get_all_indexes) {
+                my $note = $list_obj->fetch_with_id($idx)->annotation ;
+                $$data_r .= "\n$pad$element:$idx#".note_quote($note) if $note ;
+            }
 	    # skip undef values
 	    my @val = quote( grep (defined $_, 
 				   $list_obj->fetch_all_values(mode => $fetch_mode, 
 							       check => $check))) ;
-	    my $note = quote($list_obj->annotation) ;
-            $$data_r .= "\n$pad$element"        if @val or $note ;
-	    $$data_r .= "=" . join( ',', @val ) if @val;
-	    $$data_r .= '#'  . $note            if         $note ;
+            $$data_r .= "\n$pad$element=" . join( ',', @val ) if @val;
         }
     };
 
@@ -277,7 +307,7 @@ sub dump_tree {
 	my $hash_obj = $node->fetch_element($element) ;
 
 	# add annotation for list or hash element
-	my $note  = quote($hash_obj->annotation) ;
+	my $note  = note_quote($hash_obj->annotation) ;
 	$$data_r .= "\n$pad$element#$note" if $note ;
 
 	# resume exploration
@@ -302,7 +332,7 @@ sub dump_tree {
 	# but foo#comment:bar if not valid -> foo#commaent foo:bar
 
 	my $head = "\n$pad$element";
-        my $node_note = quote($contained_node->annotation) ;
+        my $node_note = note_quote($contained_node->annotation) ;
 	
 	if ($type eq 'list' or $type eq 'hash') {
 	    $head .= ':'.quote($key) ;
@@ -339,7 +369,7 @@ sub dump_tree {
     my $view_scanner = Config::Model::ObjTreeScanner->new(@scan_args);
 
     my $ret = '';
-    my $root_note = quote($node->annotation) ;
+    my $root_note = note_quote($node->annotation) ;
     $ret .="\n#$root_note" if $root_note ;
     $view_scanner->scan_node(\$ret, $node);
 
@@ -356,4 +386,4 @@ Dominique Dumont, (ddumont at cpan dot org)
 
 =head1 SEE ALSO
 
-L<Config::Model>,L<Config::Model::Node>,L<Config::Model::Walker>
\ No newline at end of file
+L<Config::Model>,L<Config::Model::Node>,L<Config::Model::Walker>
