@@ -1,7 +1,7 @@
 #
 # This file is part of Config-Model
 #
-# This software is Copyright (c) 2010 by Dominique Dumont, Krzysztof Tyszecki.
+# This software is Copyright (c) 2011 by Dominique Dumont, Krzysztof Tyszecki.
 #
 # This is free software, licensed under:
 #
@@ -27,8 +27,8 @@
 #    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 
 package Config::Model::IdElementReference ;
-BEGIN {
-  $Config::Model::IdElementReference::VERSION = '1.226';
+{
+  $Config::Model::IdElementReference::VERSION = '1.260';
 }
 
 use warnings ;
@@ -37,48 +37,117 @@ use strict;
 use Scalar::Util qw(weaken) ;
 use Carp ;
 
-
-
-
 =head1 NAME
 
 Config::Model::IdElementReference - Refer to id element(s) and extract keys
 
 =head1 VERSION
 
-version 1.226
+version 1.260
 
 =head1 SYNOPSIS
 
- # used from a value class
- element => [
-            node_host => { type => 'leaf',
-			   value_type => 'reference' ,
-			   refer_to => '! host'
-			 },
-            if   => { type => 'leaf',
-                      value_type => 'reference' ,
-                      computed_refer_to 
-                      => { formula => '  ! host:$h if ',
-                           variables => { h => '- node_host' }
-                         }
-                    },
-            ],
+ # synopsis shows an example of model of a network to use references
+ 
+ use Config::Model;
+ use Log::Log4perl qw(:easy);
+ Log::Log4perl->easy_init($WARN);
 
-  # used from checklist
-  element => [
-	      # simple reference, checklist items are given by the
-	      # keys of my_hash
-	      refer_to_list => { type => 'check_list',
-				 refer_to => '- my_hash'
-			       },
+ my $model = Config::Model->new;
 
-	      # checklist items are given by combining my_hash*
-	      refer_to_2_list
-                            => { type => 'check_list',
-				 refer_to => '- my_hash + - my_hash2   + - my_hash3'
-			       },
-             ]
+ # model of several hosts with several NICs
+ $model->create_config_class(
+    name      => 'Host',
+    'element' => [
+        ip_nic => {
+            type       => 'hash',
+            index_type => 'string',
+            cargo      => {
+                type       => 'leaf',
+                value_type => 'uniline',
+            }
+        },
+    ]
+ );
+
+ # model to choose a master host and a master NIC (whatever that may be)
+ # among configured hosts. Once these 2 are configured, the model computes 
+ # the master IP
+
+ $model->create_config_class(
+    name => "MyNetwork",
+
+    element => [
+        host => {
+            type       => 'hash',
+            index_type => 'string',
+            cargo      => {
+                type              => 'node',
+                config_class_name => 'Host'
+            },
+        },
+
+        # master_host is one of the configured hosts
+        master_host => {
+            type       => 'leaf',
+            value_type => 'reference', # provided by tConfig::Model::IdElementReference
+            refer_to   => '! host'
+        },
+
+        # master_nic is one NIC of the master host
+        master_nic => {
+            type              => 'leaf',
+            value_type        => 'reference', # provided by tConfig::Model::IdElementReference
+            computed_refer_to => {            # provided by Config::Model::ValueComputer
+                formula   => '  ! host:$h ip_nic ',
+                variables => { h => '- master_host' }
+            }
+        },
+
+        # provided by Config::Model::ValueComputer
+        master_ip => {
+            type       => 'leaf',
+            value_type => 'string',
+            compute    => {
+                formula   => '$ip',
+                variables => {
+                    h   => '- master_host',
+                    nic => '- master_nic',
+                    ip  => '! host:$h ip_nic:$nic'
+                }
+            }
+        },
+
+    ],
+ );
+
+ my $inst = $model->instance(root_class_name => 'MyNetwork' );
+
+ my $root = $inst->config_root ;
+
+ # configure hosts on my network
+ my $step = 'host:foo ip_nic:eth0=192.168.0.1 ip_nic:eth1=192.168.1.1 -
+             host:bar ip_nic:eth0=192.168.0.2 ip_nic:eth1=192.168.1.2 -
+             host:baz ip_nic:eth0=192.168.0.3 ip_nic:eth1=192.168.1.3 ';
+ $root->load( step => $step );
+
+ print "master host can be one of ",
+   join(' ',$root->fetch_element('master_host')->get_choice),"\n" ; 
+ # prints: master host can be one of bar baz foo
+
+ # choose master host
+ $root->load('master_host=bar') ;
+
+ print "master NIC of master host can be one of ",
+ join(' ',$root->fetch_element('master_nic')->get_choice),"\n" ; 
+ # prints: master NIC of master host can be one of eth0 eth1
+
+ # choose master nic
+ $root->load('master_nic=eth1') ;
+
+ # check what is the master IP computed by the model
+ print "master IP is ",$root->grab_value('master_ip'),"\n";
+ # prints master IP is 192.168.1.2
 
 
 =head1 DESCRIPTION
@@ -89,11 +158,11 @@ L<Config::Model::HashId> or the content of a L<Config::Model::ListId>
 object.
 
 This class is also used by L<Config::Model::CheckList> to define the
-cheklist items from the keys of another hash (or content of a list).
+checklist items from the keys of another hash (or content of a list).
 
 =head1 CONSTRUCTOR
 
-Construction is handled by the calling object. 
+Construction is handled by the calling object (L<Config::Model::Node>). 
 
 =cut
 
@@ -250,11 +319,12 @@ sub get_choice_from_refered_to {
 
 	if ($@) {
 	    my $e = $@ ;
-	    my $msg = $e ? $e->full_message : '' ;
+	    # don't use $e->full_description as it will recurse badly
+	    my $msg = $e ? $e->description : '' ;
 	    Config::Model::Exception::Model
 		-> throw (
 			  object => $config_elt,
-			  error => "'refer_to' parameter: " . $msg
+			  error => "'refer_to' parameter with path '@path': " . $msg
 			 ) ;
 	}
 
@@ -360,4 +430,4 @@ L<Config::Model::AnyId>, L<Config::Model::CheckList>
 
 =cut
 
-1;
\ No newline at end of file
+1;
