@@ -1,13 +1,13 @@
 #
 # This file is part of Config-Model
 #
-# This software is Copyright (c) 2010 by Dominique Dumont, Krzysztof Tyszecki.
+# This software is Copyright (c) 2011 by Dominique Dumont, Krzysztof Tyszecki.
 #
 # This is free software, licensed under:
 #
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
-#    Copyright (c) 2005-2010 Dominique Dumont.
+#    Copyright (c) 2005-2011 Dominique Dumont.
 #
 #    This file is part of Config-Model.
 #
@@ -26,8 +26,8 @@
 #    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 
 package Config::Model::Instance;
-BEGIN {
-  $Config::Model::Instance::VERSION = '1.226';
+{
+  $Config::Model::Instance::VERSION = '1.260';
 }
 use Scalar::Util qw(weaken) ;
 use File::Path;
@@ -37,8 +37,8 @@ use Config::Model::Annotation;
 use Config::Model::Exception ;
 use Config::Model::Node ;
 use Config::Model::Loader;
-use Config::Model::Searcher;
-use Config::Model::WizardHelper;
+use Config::Model::SearchElement;
+use Config::Model::Iterator;
 
 use strict ;
 use Carp;
@@ -56,15 +56,41 @@ Config::Model::Instance - Instance of configuration tree
 
 =head1 VERSION
 
-version 1.226
+version 1.260
 
 =head1 SYNOPSIS
 
- my $model = Config::Model->new() ;
- $model ->create_config_class ( ... ) ;
+ use Config::Model;
+ use Log::Log4perl qw(:easy);
+ use File::Path ;
+ Log::Log4perl->easy_init($WARN);
+
+ # setup a dummy popcon conf file
+ my $wr_dir = '/tmp/etc/';
+ my $conf_file = "$wr_dir/popularity-contest.conf" ;
+
+ unless (-d $wr_dir) {
+     mkpath($wr_dir, { mode => 0755 }) 
+       || die "can't mkpath $wr_dir: $!";
+ }
+ open(my $conf,"> $conf_file" ) || die "can't open $conf_file: $!";
+ $conf->print( qq!MY_HOSTID="aaaaaaaaaaaaaaaaaaaa"\n!,
+   qq!PARTICIPATE="yes"\n!,
+   qq!USEHTTP="yes" # always http\n!,
+   qq!DAY="6"\n!);
+ $conf->close ;
+
+ my $model = Config::Model->new;
+
+ # PopCon model is provided. Create a new Config::Model::Instance object
+ my $inst = $model->instance (root_class_name   => 'PopCon',
+                              root_dir          => '/tmp',
+                             );
+ my $root = $inst -> config_root ;
+
+ print $root->describe;
+
 
- my $inst = $model->instance (root_class_name => 'SomeRootClass', 
-                              instance_name    => 'some_name');
 
 =head1 DESCRIPTION
 
@@ -98,13 +124,18 @@ Specify which backend to use. See L</write_back ( ... )> for details
 When set, configuration files will not be read when creating
 configuration tree.
 
+=item check
+
+'yes', 'skip' or 'no'
+
 =back
 
 Note that the root directory specified within the configuration model
 will be overridden by C<root_dir> parameter.
 
 If you need to load configuration data that are not correct, you can
-use C<< force_load => 1 >>. Then, wrong data will be discarded.
+use C<< force_load => 1 >>. Then, wrong data will be discarded (equivalent to 
+C<check => 'no'> ).
 
 =cut
 
@@ -122,7 +153,9 @@ sub new {
     confess __PACKAGE__," error: config_model is not a Config::Model object"
       unless $config_model->isa('Config::Model') ; 
 
-    my $force_load = delete $args{force_load} || 0 ;
+    my $read_check = delete $args{check} || 'yes' ;
+    carp "instance new: force_load is deprecated" if defined $args{force_load} ;
+    $read_check = 'no' if delete $args{force_load} ;
 
     my $self 
       = {
@@ -137,6 +170,8 @@ sub new {
 	 safe => {
 		 } ,
 
+        read_check => $read_check ,
+
 	 # preset mode to load values found by HW scan or other
 	 # automatic scheme
 	 preset => 0,
@@ -168,7 +203,7 @@ sub new {
 
     bless $self, $class;
 
-    $self->reset_config(check => $force_load ? 'no' : 'yes' ) ;
+    $self->reset_config() ;
 
     return $self ;
 }
@@ -196,6 +231,16 @@ sub config_root {
     return shift->{tree} ;
 }
 
+=head2 read_check()
+
+Returns how to check read files.
+
+=cut
+
+sub read_check {
+    return shift->{read_check} ;
+}
+
 =head2 reset_config
 
 Destroy current configuration tree (with data) and returns a new tree with
@@ -211,7 +256,6 @@ sub reset_config {
 	       instance => $self,
 	       config_model => $self->{config_model},
 	       skip_read  => $self->{skip_read},
-	       check => $args{check} ,
 	     );
 
     # $self->{annotation_saver} = Config::Model::Annotation
@@ -333,25 +377,35 @@ sub search_element {
 
 =head2 wizard_helper ( ... )
 
-This method returns a L<Config::Model::WizardHelper> object. See
-L<Config::Model::WizardHelper> for details on how to create a wizard
-widget with this object.
-
-wizard_helper arguments are explained in  L<Config::Model::WizardHelper>
-L<constructor arguments|Config::Model::WizardHelper/"Creating a wizard helper">.
+Deprecated. Call L</iterator> instead.
 
 =cut
 
 sub wizard_helper {
+    carp __PACKAGE__,"::wizard_helper helped is deprecated. Call iterator instead" ;
+    goto &iterator ;
+}
+
+=head2 iterator 
+
+This method returns a L<Config::Model::Iterator> object. See
+L<Config::Model::Iterator> for details.
+
+Arguments are explained in  L<Config::Model::Iterator>
+L<constructor arguments|Config::Model::Iterator/"Creating an iterator">.
+
+=cut
+
+sub iterator {
     my $self = shift ;
     my @args = @_ ;
 
     my $tree_root = $self->config_root ;
 
-    return Config::Model::WizardHelper->new ( root => $tree_root, @args) ;
+    return Config::Model::Iterator->new ( root => $tree_root, @args) ;
 }
 
-
+=head2 
 
 =head1 Auto read and write feature
 
@@ -405,19 +459,18 @@ sub write_root_dir {
     return $self -> {root_dir} ;
 }
 
-=head2 register_write_back ( backend_name, sub_ref )
+=head2 register_write_back ( node_location )
 
-Register a sub ref (with the backend name) that will be called with
+Register a node path that will be called back with
 C<write_back> method.
 
 =cut
 
 sub register_write_back {
-    my ($self,$backend,$wb) = @_ ;
+    my ($self,$node_path) = @_ ;
+    $logger->debug("register_write_back: instance '$self->{name}' registers node '$node_path'") ;
 
-    croak "register_write_back: parameter is not a code ref"
-      unless ref($wb) eq 'CODE' ;
-    push @{$self->{write_back}}, [$backend, $wb] ;
+    push @{$self->{write_back}}, $node_path ;
 }
 
 =head2 write_back ( ... )
@@ -426,9 +479,9 @@ Try to run all subroutines registered with C<register_write_back> to
 write the configuration information until one succeeds (returns
 true). (See L<Config::Model::AutoRead> for details).
 
-You can specify here a pseudo root dir or another config dir to write
-configuration data back with C<root> and C<config_dir> parameters. This
-will override the model specifications.
+You can specify here a pseudo root directory or another config
+directory to write configuration data back with C<root> and
+C<config_dir> parameters. This will override the model specifications.
 
 You can force to use a backend by specifying C<< backend => xxx >>. 
 For instance, C<< backend => 'augeas' >> or C<< backend => 'custom' >>.
@@ -455,25 +508,63 @@ sub write_back {
      }
       keys %args;
 
-    croak "write_back: no subs registered. cannot save data\n" 
+    croak "write_back: no subs registered in instance $self->{name}. cannot save data\n" 
       unless @{$self->{write_back}} ;
 
-    my $dir = $args{config_dir} ;
-    mkpath($dir,0,0755) if $dir and not -d $dir ;
-
-    foreach my $wb_info (@{$self->{write_back}}) {
-	my ($backend,$wb) = @$wb_info ;
-	if (not $force_backend 
-	    or  $force_backend eq $backend 
-	    or  $force_backend eq 'all' ) {
-	    # exit when write is successfull
-	    my $res = $wb->(%args) ; 
-	    $logger->info("write_back called with $backend backend, result is ", defined $res ? $res : '<undef>' );
-	    last if ($res and not $force_backend); 
-	}
+    foreach my $path (@{$self->{write_back}}) {
+	$logger->info("write_back called on node $path");
+        my $node = $self->config_root->grab(step => $path, type => 'node');
+        $node->write_back(%args, backend => $force_backend);
     }
 }
 
+=head2 apply_fixes
+
+Scan the tree and apply fixes that are attached to warning specifications. 
+See C<warn_if_match> or C<warn_unless_match> in L<Config::Model::Value/>.
+
+=cut
+
+sub apply_fixes {
+    my $self = shift ;
+
+    # define leaf call back
+    my $fix_leaf = sub { 
+      my ($scanner, $data_ref, $node,$element_name,$index, $leaf_object) = @_ ;
+      $leaf_object->apply_fixes ;
+    } ;
+
+    my $fix_hash = sub {
+        my ( $scanner, $data_r, $node, $element, @keys ) = @_;
+
+        return unless @keys;
+
+        # leaves must be fixed before the hash, hence the 
+        # calls to scan_hash before apply_fixes
+        map {$scanner->scan_hash($data_r,$node,$element,$_)} @keys ;
+
+        $node->fetch_element($element)->apply_fixes ;
+    } ;
+    
+    my $fix_list = sub {
+        my ( $scanner, $data_r, $node, $element, @keys ) = @_;
+
+        return unless @keys;
+
+        map {$scanner->scan_list($data_r,$node,$element,$_)} @keys ;
+        $node->fetch_element($element)->apply_fixes ;
+    } ;
+    
+   my $scan = Config::Model::ObjTreeScanner-> new ( 
+        hash_element_cb => $fix_hash ,
+        list_element_cb => $fix_list ,
+        leaf_cb => $fix_leaf ,
+        check => 'no',
+    ) ;
+
+    $scan->scan_node(undef, $self->config_root) ;
+}
+
 sub push_no_value_check { carp "push_no_value_check is deprecated";}
 sub pop_no_value_check  { carp "pop_no_value_check is deprecated";}
 sub get_value_check { carp "get_value_check is deprecated";}
@@ -492,4 +583,4 @@ L<Config::Model::Loader>,
 L<Config::Model::Searcher>,
 L<Config::Model::Value>,
 
-=cut
\ No newline at end of file
+=cut
