@@ -1,13 +1,13 @@
 #
 # This file is part of Config-Model
 #
-# This software is Copyright (c) 2010 by Dominique Dumont, Krzysztof Tyszecki.
+# This software is Copyright (c) 2011 by Dominique Dumont, Krzysztof Tyszecki.
 #
 # This is free software, licensed under:
 #
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
-#    Copyright (c) 2005-2010 Dominique Dumont, Krzysztof Tyszecki.
+#    Copyright (c) 2005-2011 Dominique Dumont, Krzysztof Tyszecki.
 #
 #    This file is part of Config-Model.
 #
@@ -26,8 +26,8 @@
 #    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 
 package Config::Model::Node;
-BEGIN {
-  $Config::Model::Node::VERSION = '1.226';
+{
+  $Config::Model::Node::VERSION = '1.260';
 }
 use Carp ;
 use strict;
@@ -37,13 +37,15 @@ use Config::Model::Loader;
 use Config::Model::Dumper;
 use Config::Model::DumpAsData;
 use Config::Model::Report;
+use Config::Model::TreeSearcher;
 use Config::Model::Describe;
+use Config::Model::BackendMgr;
 use Log::Log4perl qw(get_logger :levels);
 use UNIVERSAL;
 use Scalar::Util qw/weaken/;
 use Storable qw/dclone/ ;
 
-use base qw/Config::Model::AutoRead/;
+use base qw/Config::Model::AnyThing/;
 
 use vars qw(@status @level
             @experience_list %experience_index %default_property);
@@ -68,42 +70,66 @@ Config::Model::Node - Class for configuration tree node
 
 =head1 VERSION
 
-version 1.226
+version 1.260
 
 =head1 SYNOPSIS
 
- $model->create_config_class 
-  (
-   name              => 'OneConfigClass',
-   class_description => "OneConfigClass detailed description",
+ use Config::Model;
+ use Log::Log4perl qw(:easy);
+ Log::Log4perl->easy_init($WARN);
 
-   element           => [
-                          [qw/X Y Z/] 
-                          => {
-                               type => 'leaf',
-                               value_type => 'enum',
-                               choice     => [qw/Av Bv Cv/]
-                             }
-                        ],
+ # define configuration tree object
+ my $model = Config::Model->new;
+ $model->create_config_class(
+    name              => 'OneConfigClass',
+    class_description => "OneConfigClass detailed description",
 
-   experience        => [ Y => 'beginner', 
-                          X => 'master' 
-                        ],
-   status            => [ X => 'deprecated' ],
-   description       => [ X => 'X-ray description (can be long)' ],
-   summary           => [ X => 'X-ray' ],
+    element => [
+        [qw/X Y Z/] => {
+            type       => 'leaf',
+            value_type => 'enum',
+            choice     => [qw/Av Bv Cv/]
+        }
+    ],
 
-   accept            => [ { name_match => 'ip.*',
-                            type => 'leaf',
-                            value_type => 'uniline',
-                            summary => 'ip address',
-                          }
-                        ]
-  );
+    experience => [
+        Y => 'beginner',
+        X => 'master'
+    ],
+    status      => [ X => 'deprecated' ],
+    description => [ X => 'X-ray description (can be long)' ],
+    summary     => [ X => 'X-ray' ],
 
- my $instance = $model->instance (root_class_name => 'OneConfigClass', 
-                                  instance_name => 'test1');
- my $root_node = $instance -> config_root ;
+    accept => [
+        'ip.*' => {
+            type       => 'leaf',
+            value_type => 'uniline',
+            summary    => 'ip address',
+        }
+    ]
+ );
+ my $instance = $model->instance (root_class_name => 'OneConfigClass');
+ my $root = $instance->config_root ;
+
+ # X is not shown below because of its deprecated status
+ print $root->describe,"\n" ;
+ # name         value        type         comment
+ # Y            [undef]      enum         choice: Av Bv Cv
+ # Z            [undef]      enum         choice: Av Bv Cv
+
+ # add some data
+ $root->load( step => 'Y=Av' );
+
+ # add some accepted element, ipA and ipB are created on the fly
+ $root->load( step => q!ipA=192.168.1.0 ipB=192.168.1.1"! );
+
+ # show also ip* element created in the last "load" call
+ print $root->describe,"\n" ;
+ # name         value        type         comment
+ # Y            Av           enum         choice: Av Bv Cv
+ # Z            [undef]      enum         choice: Av Bv Cv
+ # ipA          192.168.1.0  uniline
+ # ipB          192.168.1.1  uniline
 
 =head1 DESCRIPTION
 
@@ -223,29 +249,28 @@ See L<Config::Model::AutoRead> for details.
 
 =item B<accept>
 
-Optional list of criteria to accept unknown parameters. Each criteria will have
-a list of specification that will enable C<Config::Model> to create a model
+Optional list of criteria (i.e. a regular expression to match ) to
+accept unknown parameters. Each criteria will have a list of
+specification that will enable C<Config::Model> to create a model
 snippet for the unknown element.
 
 Example:
 
-  accept =>      [
-                    {
-                            name_match => 'list.*',
-                            type => 'list',
-                            cargo => {
-                                        type => 'leaf',
-                                        value_type => 'string',
-                                     } ,
-                     },
-                     {
-                            name_match => 'str.*',
-                            type => 'leaf',
-                            value_type => 'uniline'
-                     },
-                  ]
+ accept => [
+    'list.*' => {
+        type  => 'list',
+        cargo => {
+            type       => 'leaf',
+            value_type => 'string',
+        },
+    },
+    'str.*' => {
+        type       => 'leaf',
+        value_type => 'uniline'
+    },
+  ]
 
-All C<element> parameter and C<name_match> can be used for C<accept>.
+All C<element> parameters can be used in specifying accepted parameters.
 
 =for html
 <p>For more information, see <a href="http://ddumont.wordpress.com/2010/05/19/improve-config-upgrade-ep-02-minimal-model-for-opensshs-sshd_config/">this blog<\a>.</p>
@@ -321,7 +346,7 @@ sub create_element {
       " error: no create method for element type $element_info->{type}"
         unless defined $method ;
 
-    $self->$method($element_name) ;
+    $self->$method($element_name, $check) ;
 }
 
 
@@ -330,7 +355,7 @@ sub create_element {
 
 =item C<node>
 
-The element is a simple node of a tree instanciated from a 
+The element is a simple node of a tree instantiated from a 
 configuration class (declared with 
 L<Config::Model/"create_config_class( ... )">). 
 See L</"Node element">.
@@ -338,8 +363,7 @@ See L</"Node element">.
 =cut
 
 sub create_node {
-    my $self= shift ;
-    my $element_name = shift ;
+    my ($self, $element_name, $check)  = @_ ;
 
     my $element_info = dclone($self->{model}{element}{$element_name}) ; 
 
@@ -353,7 +377,9 @@ sub create_node {
 
     my @args = (config_class_name => $element_info->{config_class_name},
                 instance          => $self->{instance},
-                element_name      => $element_name) ;
+                element_name      => $element_name ,
+                check             => $check ,
+    ) ;
 
     $self->{element}{$element_name} = $self->new(@args) ;
 }
@@ -368,15 +394,15 @@ for details.
 =cut
 
 sub create_warped_node {
-    my $self= shift ;
-    my $element_name = shift ;
+    my ($self, $element_name, $check)  = @_ ;
 
     my $element_info = dclone($self->{model}{element}{$element_name}) ; 
 
     my @args = (instance          => $self->{instance},
                 element_name      => $element_name,
-                parent            => $self
-               ) ;
+                parent            => $self,
+                check             => $check,
+    ) ;
 
     require Config::Model::WarpedNode ;
 
@@ -391,8 +417,7 @@ The element is a scalar value. See L</"Leaf element">
 =cut
 
 sub create_leaf {
-    my $self = shift ;
-    my $element_name = shift ;
+    my ($self, $element_name, $check)  = @_ ;
 
     my $element_info = dclone($self->{model}{element}{$element_name}) ; 
 
@@ -442,8 +467,7 @@ my %id_class_hash = (
                     ) ;
 
 sub create_id {
-    my $self = shift ;
-    my $element_name = shift ;
+    my ($self, $element_name, $check)  = @_ ;
 
     my $element_info = dclone($self->{model}{element}{$element_name}) ; 
     my $type = delete $element_info->{type} ;
@@ -592,7 +616,7 @@ sub new {
     my $caller = shift;
     my $type = ref($caller) || $caller ;
 
-    my $self         = {};
+    my $self         = { initialized => 0 };
     bless $self, $type;
 
     my @mandatory_parameters = qw/config_class_name instance/;
@@ -618,8 +642,16 @@ sub new {
 
     $self->{index_value} = delete $args{index_value} ;
     my $skip_read = delete $args{skip_read} ;
-    my $check = $self->_check_check(delete $args{check}) ;
-
+    my $req_check = $self->_check_check(delete $args{check}) ;
+    my $read_check = $self->instance->read_check;
+    
+    my $check = $req_check eq 'no'   || $read_check eq 'no'   ? 'no'
+              : $req_check eq 'skip' || $read_check eq 'skip' ? 'skip' 
+              :                                                 'yes' ;
+    
+    
+    $self->{auto_read} = { skip => delete $args{skip_read}, check => $check };
+    
     my @left = keys %args ;
     croak "Node->new: unexpected parameter: @left" if @left ;
 
@@ -636,11 +668,27 @@ sub new {
         
     $self->check_properties ;
 
-    if (defined $model->{read_config} and not $skip_read) {
+    return $self ;
+}
+
+sub init {
+    my $self = shift ;
+
+    return if $self->{initialized} ;
+    $self->{initialized} = 1 ; # avoid recursions
+
+    my $model = $self->{model} ;
+    
+    return unless defined $model->{read_config} or defined $model->{write_config} ;
+    $self->{bmgr} ||= Config::Model::BackendMgr->new (node => $self) ;
+    
+    my $ar = $self->{auto_read} ;
+    
+    my $check = $ar->{check} ;
+    if (defined $model->{read_config} and not $ar->{skip_read} ) {
+        $ar->{done} = 1 ;
         # setup auto_read, read_config_dir is obsolete
-        $self->auto_read_init($model->{read_config}, $check,
-                              $model->{read_config_dir}
-                              );
+        $self->{bmgr}->auto_read_init($model->{read_config}, $check, $model->{read_config_dir} );
     }
 
     # use read_config data if write_config is missing
@@ -649,11 +697,20 @@ sub new {
 
     if ($model->{write_config}) {
         # setup auto_write, write_config_dir is obsolete
-        $self->auto_write_init($model->{write_config},
-                               $model->{write_config_dir});
+        $self->{bmgr}->auto_write_init($model->{write_config},
+                                       $model->{write_config_dir});
     }
+}
 
-    return $self ;
+sub write_back {
+    my $self = shift ;
+    $self->{bmgr}->write_back(@_) ;
+}
+
+sub is_auto_write_for_type {
+    my $self = shift ;
+    return 0 unless defined $self->{bmgr} ;
+    return $self->{bmgr}->is_auto_write_for_type(@_) ;
 }
 
 =head1 Introspection methods
@@ -717,20 +774,26 @@ for my $datum (qw/config_model model config_class_name/) {
     } ;
 }
 
-=head2 has_element ( element_name )
+=head2 has_element ( name => element_name, [ type => searched_type ] )
 
 Returns 1 if the class model has the element declared or if the element 
-name is matched by the optional C<accept> parameter. 
+name is matched by the optional C<accept> parameter. If C<type> is specified, the 
+element name must also match the type.
 
 =cut
 
 # should I autovivify this element: NO
 sub has_element {
-    my ($self,$name) = @_ ;
+    my $self = shift ;
+    my %args = ( @_ > 1 ) ? @_ : ( name => shift ) ;
+    my $name = $args{name};
+    my $type = $args{type} ;
     croak "has_element: missing element name" unless defined $name ;
 
     $self->accept_element($name);
-    return defined $self->{model}{element}{$name} ? 1 : 0 ;
+    return 0 unless defined $self->{model}{element}{$name} ;
+    return 1 unless defined $type ;
+    return $self->{model}{element}{$name}{type} eq $type ? 1 : 0 ;
 }
 
 =head2 find_element ( element_name , [ case => any ])
@@ -750,25 +813,27 @@ sub find_element {
     my ($self,$name, %args ) = @_ ;
     croak "find_element: missing element name" unless defined $name ;
 
-    $self->accept_element($name);
-    return $name if defined $self->{model}{element}{$name} ;
-
-    # now look for a close element playing with cases;
+    # look for a close element playing with cases;
     if (defined $args{case} and $args{case} eq 'any') {
         foreach my $elt (keys %{$self->{model}{element}}) {
             return $elt if lc($elt) eq lc ($name) ;
         }
     }
+
+    # now look if the element can be accepted
+    $self->accept_element($name);
+    return $name if defined $self->{model}{element}{$name} ;
+
     return ;
 }
 
-=head2 searcher ()
+=head2 model_searcher ()
 
 Returns an object dedicated to search an element in the configuration
 model (respecting privilege level).
 
-This method returns a L<Config::Model::Searcher> object. See
-L<Config::Model::Searcher> for details on how to handle a search.
+This method returns a L<Config::Model::SearchElement> object. See
+L<Config::Model::SearchElement> for details on how to handle a search.
 
 This method is inherited from L<Config::Model::AnyThing>.
 
@@ -836,9 +901,9 @@ See L<Config::Model::AnyThing/"location()">
 
 Return all elements names available for C<experience>.
 If no experience is specified, will return all
-slots available at 'master' level (I.e all elements).
+elements available at 'master' level (I.e all elements).
 
-Optional paremeters are:
+Optional parameters are:
 
 =over
 
@@ -855,6 +920,10 @@ C<get_element_name> will return simple leaf elements, but also hash
 or list element that contain L<leaf|Config::Model::Value> object. By
 default return elements of any type.
 
+=item *
+
+B<check>: C<yes>, C<no> or C<skip>
+
 =back
 
 Returns an array in array context, and a string 
@@ -879,6 +948,8 @@ sub get_element_name {
       join (' or ', @experience_list) 
         unless defined $experience_index{$for} ;
 
+    $self->init ;
+
     my $for_idx = $experience_index{$for} ;
 
     my @result ;
@@ -892,7 +963,8 @@ sub get_element_name {
     foreach my $elt (@element_list) {
         # create element if they don't exist, this enables warp stuff
         # to kick in
-        $self->create_element($elt) unless defined $self->{element}{$elt};
+        $self->create_element(name => $elt, check => $args{check} || 'yes') 
+            unless defined $self->{element}{$elt};
 
         next if $self->{level}{$elt} eq 'hidden' ;
 
@@ -928,23 +1000,25 @@ sub children {
     return $self-> get_element_name ;
 }
 
-=head2 next_element ( element_name, [ experience_index ] )
+=head2 next_element ( ... )
 
 This method provides a way to iterate through the elements of a node.
+Mandatory parameter is C<name>. Optional parameters are C<experience>
+and C<status>.
 
 Returns the next element name for a given experience (default
-C<master>).  Returns undef if no next element is available.
+C<master>) and status (default C<normal>).
+Returns undef if no next element is available.
 
 =cut
 
 sub next_element {
     my $self      = shift;
-    my $element   = shift;
-    my $min_experience = shift;
-    my $find_previous = shift || 0 ;
+    my %args = @_ ;
+    my $element   = $args{name} ;
 
     my @elements = @{$self->{model}{element_list}} ;
-    @elements = reverse @elements if $find_previous ;
+    @elements = reverse @elements if $args{reverse} ;
 
     # if element is empty, start from first element
     my $found_elt = (defined $element and $element) ? 0 : 1 ;
@@ -953,9 +1027,10 @@ sub next_element {
         if ($found_elt) {
             return $name 
               if $self->is_element_available(name => $name, 
-                                             experience => $min_experience);
+                                             experience => $args{experience},
+                                             status => $args{status});
         }
-        $found_elt = 1 if $element eq $name ;
+        $found_elt = 1 if defined $element and $element eq $name ;
     }
 
     croak "next_element: element $element is unknown. Expected @elements" 
@@ -963,7 +1038,7 @@ sub next_element {
     return;
 }
 
-=head2 previous_element ( element_name, [ experience_index ] )
+=head2 previous_element ( name => element_name, [ experience => min_experience ] )
 
 This method provides a way to iterate through the elements of a node.
 
@@ -974,9 +1049,7 @@ C<master>).  Returns undef if no previous element is available.
 
 sub previous_element {
     my $self      = shift;
-    my $element   = shift;
-    my $min_experience = shift;
-    $self->next_element($element,$min_experience,1) ;
+    $self->next_element(@_, reverse => 1) ;
 }
 
 =head2 get_element_property ( element => ..., property => ... )
@@ -1107,6 +1180,8 @@ sub fetch_element {
         $user = 'beginner' ;
     }
 
+    $self->init($check);
+
     my $model = $self->{model} ;
 
     # retrieve element (and auto-vivify if needed)
@@ -1243,13 +1318,14 @@ As a syntactic sugar, this method can be called with only one parameter:
 
 sub is_element_available {
     my $self = shift;
-    my ($elt_name, $user_experience) = (undef, 'beginner');
+    my ($elt_name, $user_experience,$status) = (undef, 'beginner','deprecated');
     if (@_ == 1) {
         $elt_name = shift ;
     } else {
         my %args = @_ ;
         $elt_name = $args{name} ;
         $user_experience = $args{experience} if defined $args{experience} ;
+        $status = $args{status} if defined $args{status} ;
         if (defined $args{permission}) {
             $user_experience = $args{permission};
             carp "is_element_available: permission is deprecated" ;
@@ -1268,6 +1344,11 @@ sub is_element_available {
                                                     element => $elt_name) ;
     return 0 if $element_level eq 'hidden' ;
 
+    my $element_status = $self->get_element_property(property => 'status',
+                                                    element => $elt_name) ;
+
+    return 0 unless ($element_status eq 'standard' or $element_status eq $status) ;
+
     my $element_exp = $self->get_element_property(property => 'experience',
                                                   element => $elt_name) ;
 
@@ -1287,7 +1368,7 @@ sub is_element_available {
 =head2 accept_element( name )
 
 Checks and returns the appropriate model of an acceptable element 
-(be it explicetely declared, or part of an C<accept> declaration).
+(be it explicitly declared, or part of an C<accept> declaration).
 Returns undef if the element cannot be accepted.
 
 =cut
@@ -1429,7 +1510,7 @@ sub set {
     }
 }
 
-=head1 Serialisation
+=head1 Serialization
 
 =head2 load ( step => string [, experience => ... ] )
 
@@ -1464,30 +1545,23 @@ sub load {
     }
 }
 
-=head2 load_data ( hash_ref, hash_ref,[ $check  ])
+=head2 load_data ( hash_ref, [ $check  ])
 
 Load configuration data with a hash ref (first parameter). The hash ref key must match
 the available elements of the node. The hash ref structure must match
 the structure of the configuration model.
 
-The second parameter is optional and contains annotations for
-elements. A standard hash of hash (or list) may contain annotation
-only for leaf elements. In order to support annotation for nodes or
-other elements, hash keys can also contain annotations like "foo# foo
-note".  Also the special key '__' will store the annotation in the
-containing object.
-
 =cut
 sub load_data {
     my $self                = shift ;
     my $raw_perl_data       = shift ;
-    my $raw_annotation_data = shift || {};
 
     my $check = $self->_check_check(shift) ;
 
     if (    not defined $raw_perl_data 
         or (ref($raw_perl_data) ne 'HASH' 
-        and not $raw_perl_data->isa( 'HASH' )) ) {
+            #and not $raw_perl_data->isa( 'HASH' )
+            ) ) {
         Config::Model::Exception::LoadData
             -> throw (
                       object => $self,
@@ -1499,35 +1573,10 @@ sub load_data {
 
 
     my $perl_data       = dclone $raw_perl_data ;
-    my $annotation_data = dclone $raw_annotation_data ;
 
     $logger->info("Node load_data (",$self->location,") will load elt ",
                   join (' ',keys %$perl_data));
 
-    # handle special '__' element to store annotation in containing node
-    # this is mostly useful for root node
-    my $node_annotation = delete $annotation_data->{__} ;
-    if (defined $node_annotation) {
-        $logger->debug("Node load_data annotation from '__': $node_annotation");
-        $self->annotation($node_annotation);
-    }
-
-    # put aside annotation to be stored later directly in elements
-    # i.e. scalar values and contained in key like "foo#comment"
-    my %elt_note ;
-
-    foreach my $k (keys %$annotation_data) {
-        my ($elt,$note) = split (/#\s*/,$k);
-        next unless $note ;
-        $elt_note{$elt} = $note ;
-        $annotation_data->{$elt} = delete $annotation_data->{$k} ;
-    }
-
-    foreach my $k (keys %$annotation_data) {
-        next if ref( $annotation_data->{$k} );
-        $elt_note{$k} = delete $annotation_data->{$k} ;
-    }
-
     # data must be loaded according to the element order defined by
     # the model. This will not load not yet accepted parameters
     foreach my $elt ( @{$self->{model}{element_list}} ) {
@@ -1540,8 +1589,7 @@ sub load_data {
             my $obj = $self->fetch_element(name => $elt, experience => 'master', 
                                            check => $check) ;
 
-            $obj -> load_data(delete $perl_data->{$elt}, 
-                              delete $annotation_data->{$elt}) ;
+            $obj -> load_data(delete $perl_data->{$elt}) ;
         } elsif ($check ne 'skip')  {
             Config::Model::Exception::LoadData 
                 -> throw (
@@ -1562,19 +1610,10 @@ sub load_data {
             #TODO: annotations
             my $obj = $self->fetch_element(name => $elt, experience => 'master', check => $check) ;
             $logger->debug("Node load_data: accepting element $elt");
-            $obj ->load_data(delete $perl_data->{$elt}, 
-                             delete $annotation_data->{$elt}
-                             ) if defined $obj;
+            $obj ->load_data(delete $perl_data->{$elt}) if defined $obj;
             }
     }
 
-    # now load annotations that were put aside
-    foreach my $elt (keys %elt_note) {
-        my $obj = $self->fetch_element(name => $elt, experience => 'master', check => $check) ;
-        $logger->debug("Node load_data: store element $elt annotation: $elt_note{$elt}");
-        $obj -> annotation($elt_note{$elt}) if defined $obj;
-        }
-
     if (%$perl_data and $check eq 'yes') {
         Config::Model::Exception::LoadData 
             -> throw (
@@ -1603,18 +1642,35 @@ passed to C<load>.
 
 sub dump_tree {
     my $self = shift ;
+    $self->init ;
     my $dumper = Config::Model::Dumper->new ;
     $dumper->dump_tree(node => $self, @_) ;
 }
 
+=head2 dump_annotations_as_pod ( ... )
+
+Dumps the configuration annotations of the node and its siblings into a
+string.  See L<Config::Model::Dumper/dump_annotations_as_pod> for parameter details.
+
+=cut
+
+sub dump_annotations_as_pod {
+    my $self = shift ;
+    $self->init ;
+    my $dumper = Config::Model::DumpAsData->new ;
+    $dumper->dump_annotations_as_pod(node => $self, @_) ;
+}
+
+
 =head2 describe ( [ element => ... ] )
 
-Provides a decription of the node elements or of one element.
+Provides a description of the node elements or of one element.
 
 =cut
 
 sub describe {
     my $self = shift ;
+    $self->init ;
 
     my $descriptor = Config::Model::Describe->new ;
     $descriptor->describe(node => $self, @_) ;
@@ -1629,6 +1685,7 @@ node.
 
 sub report {
     my $self = shift ;
+    $self->init ;
     my $reporter = Config::Model::Report->new ;
     $reporter->report(node => $self) ;
 }
@@ -1643,6 +1700,7 @@ value.
 
 sub audit {
     my $self = shift ;
+    $self->init ;
     my $reporter = Config::Model::Report->new ;
     $reporter->report(node => $self, audit => 1) ;
 }
@@ -1703,6 +1761,33 @@ sub get_help {
     return defined $help ? $help : '';
 }
 
+=head2 tree_searcher( type => ... )
+
+Returns an object able to search the configuration tree. 
+Parameters are :
+
+=over
+
+=item type
+
+Where to perform the search. It can be C<element>, C<value>,
+C<key>, C<summary>, C<description>, C<help> or C<all>.
+
+=back
+
+Typically, you will have to call C<search> on this object.
+
+Returns a L<Config::Model::TreeSearcher> object.
+
+
+=cut
+
+sub tree_searcher {
+    my $self = shift;
+    
+    return Config::Model::TreeSearcher->new ( node => $self, @_ );
+}
+
 1;
 
 =head2 AutoRead nodes
@@ -1711,8 +1796,6 @@ As configuration model are getting bigger, the load time of a tree
 gets longer. The L<Config::Model::AutoRead> class provides a way to
 load the configuration information only when needed.
 
-TBD
-
 =head1 AUTHOR
 
 Dominique Dumont, (ddumont at cpan dot org)
@@ -1727,4 +1810,4 @@ L<Config::Model::CheckList>,
 L<Config::Model::WarpedNode>,
 L<Config::Model::Value>
 
-=cut
\ No newline at end of file
+=cut
